/*
 * This file is part of Jelie,
 * (c) 2002 Julien Pilet <julien.pilet@epfl.ch> and
 * Stephane Magnenat <stephane.magnenat@epfl.ch>
 *
 * Jelie is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * Jelie is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

@
@ Initialize the pxa 250
@ Julien Pilet & Stephane Magnenat, 7 2002
@

#include "pxa250regs.S"

.section .text
.global _start
_start:

@ Exception vectors

@ 0x00, Reset
	b Reset
@ 0x04, Undefined instructions
	b HandleUndef
@ 0x08, Software interrupt (SWI)
	b HandleSWI
@ 0x0c, Prefetch Abort (instruction fetch memory abort)
	b HandlePrefetchAbort
@ 0x10, Data Abord (data access memory abort)
	b HandleDataAbort
@ 0x14, Reserved
	nop
@ 0x18, IRQ (interrupt)
	b HandleIRQ
@ 0x1c, FIQ (fast interrupt)
	b HandleFIQ

@ CPU jump here just after reset 
Reset:
	@ Disable watchdog, clear OS Timer
	@ OS Timer: base address 0x40A00000
	mov r0, #0x40000000
	add r0, r0, #0x00A00000
	mov r1, #0
	str r1, [r0]		@ OSMR0 = 0 Match Register 0
	str r1, [r0, #4]	@ OSMR1 = 0
	str r1, [r0, #8]	@ OSMR2 = 0
	str r1, [r0, #0xC]	@ OSMR3 = 0
	str r1, [r0, #0x10]	@ OSCR = 0 Counter Register
	str r1, [r0, #0x14]	@ OSSR = 0 Status Register 
	str r1, [r0, #0x18]	@ OWER = 0 Watchdog Enable Register
	str r1, [r0, #0x1C]	@ OIER = 0 Interupt Enable Register

	@ configuration of the GPIOs
	bl InitMachineGPIO

	@ initialisation of the serial port
	bl InitSerial

	@ Globally enable GPIO in Power Manager and Reset Control
	mov r0, #0x40000000
	add r0, r0, #0x00F00000
	orr r4, r1, r3, asl #4
	orr r4, r4, r3, asl #5
	str r4, [r0, #0x4]

	@ initialisation of the SDRAM
	bl InitMachineSDRAM

	@ jump to main application...
	b AppMain

@ Exception habndling number
HandleUndef:
	mov r5, #4
	b FatalError

HandleSWI:
	mov r5, #8
	b FatalError

HandlePrefetchAbort:
	mov r5, #0xC
	b FatalError

HandleDataAbort:
	mov r5, #0x10
	b FatalError

HandleIRQ:
	mov r5, #0x18
	b FatalError

HandleFIQ:
	mov r5, #0x1C


@ Error, print error and do not return
.global FatalError
FatalError:
	@ lr contains the address that caused the exception. Save it.
	mov r6, lr

	@ print exception number
	mov r0, #'E'
	bl SendByte
	mov r0, r5
	bl SendASCIIWord

	@ print fault address
	mov r0, #'@'
	bl SendByte
	mov r0, r6
	bl SendASCIIWord

ErrorLp:
	b ErrorLp
