/*
 * This file is part of Jelie,
 * (c) 2002 Julien Pilet <julien.pilet@epfl.ch> and
 * Stephane Magnenat <stephane.magnenat@epfl.ch>
 *
 * Jelie is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * Jelie is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 @
 @ Initialize the Korebots machine
 @

#include <pxa250regs.S>
#include <utils.S>

@ Return the machine identifier in R1
.global GetMachineId
GetMachineId:
	lreg r1, 0x146
	mov pc, lr


@ Init the GPIO relative to the machine
.global InitMachineGPIO
InitMachineGPIO:

	@ GPIO init
	@ Every GPIO in output mode
	mov r0, #0x40000000
	add r0, r0, #0x00E00000 @ r0 = GPIO base = 0x40E00000
	sub r2, r1, #1		@ r1 = 0, r2 = -1
	add r3, r1, #1		@ r3 = 1

	@ Set all pins in input, except :

	@	GPIO 15 (nCS1)
	orr r4, r1, #GPIO15_nCS1
	@	GPIO 28 (I2S BitClock)
	orr r4, r4, #GPIO28_BITCLK
	str r4, [r0, #GPDR0_OFFSET]

	@	GPIO 32 (I2S SysClock)
	orr r4, r1, #GPIO32_SYSCLK
	@	GPIO 39 (serial TXD)
	orr r4, r4, #GPIO39_FFTXD
	str r4, [r0, #GPDR1_OFFSET]

	str r1, [r0, #GPDR2_OFFSET]

	@ USB Host: Configure CS1 (GP15) to alternate function
	ldr r1, [r0, #GAFR0_L_OFFSET]
	orr r1, r1, #GP15_nCS1
	str r1, [r0, #GAFR0_L_OFFSET]
	@ I2S : Configure BitClock (GP28) to alternate
	ldr r1, [r0, #GAFR0_U_OFFSET]
	orr r1, r1, #GP28_BITCLK_I2S
	str r1, [r0, #GAFR0_U_OFFSET]
	@ I2S : Configure SysClock (GP32) to alternate
	ldr r1, [r0, #GAFR1_L_OFFSET]
	orr r1, r1, #GP32_SYSCLK
	str r1, [r0, #GAFR1_L_OFFSET]

	@ I2S : Enable output
	mov r0, #0x40000000
	add r0, r0, #0x00400000
	ldr r1, [r0, #SACR0_OFFSET]
	orr r1, r1, #SACR0_BCKD
	str r1, [r0, #SACR0_OFFSET]
	@ Note : I2S Clock is already enabled

	mov pc, lr


@ Init the SDRAM relative to the machine
.global InitMachineSDRAM
InitMachineSDRAM:

@********************************************************************************************
@** setup memory controller registers
@ ** Dev Manual sect. 5.14 step 1

@---- Wait 200 usec
	lreg r3, OSCR		@ reset the OS Timer Count to zero
	mov r2, #0
	str r2, [r3]
	ldr r4, =0x300		@ really 0x2E1 is about 200usec, so 0x300 should be plenty
t1:
	ldr r2, [r3]
	cmp r4, r2
	bgt t1

@ ** Dev Manual sect. 5.14 step 2 -- Configures /CS0
	lreg r3, MSC0
	lreg r2, 0x26C026C0
	str  r2, [r3]
	ldr  r2, [r3]		@ the MSC register should be read after it is written with
						@ a new value before an access to the memory is attempted.
						@ (see MSC section of manual)


@ ** Dev Manual sect. 5.14 step 3 -- (Skip if no SRAM)
	lreg r3, MDREFR
	lreg r2, 0x00118018
	str r2, [r3]
	nop					@ no action is required to change from "PWRDNX" to "NOP" state


@ ** Dev Manual sect. 5.14 step 4 -- (Skip if no SDRAM)
	lreg r3, MDCNFG		@ Load the SDRAM Configuration register. Must not be enabled yet.
	lreg r2, 0x00000AC8	@ Generated value
	str r2, [r3]		@ Write to MDCNFG register


@ ** Dev Manual sect. 5.14 step 5 -- (Skip if no SDRAM)
@---- Wait 200 usec
	lreg r3, OSCR		@ reset the OS Timer Count to zero
	mov r2, #0
	str r2, [r3]
	mov r4, #0x300		@ really 0x2E1 is about 200usec, so 0x300 should be plenty
t2:
	ldr r2, [r3]
	cmp r4, r2
	bgt t2


@ ** Dev Manual sect. 5.14 step 6 is skipped, we only support configuring
@ SDRAM out of RESET


@ ** Dev Manual sect. 5.14 step 7 -- (Skip if no SDRAM)
	lreg r3, 0xA0000000
	mov r2, #8			@ now must do 8 refresh or CBR commands before the first access

CBR_refresh1:
	str r3, [r3]
	subs r2, r2, #1
	bne CBR_refresh1


@ ** Dev Manual sect. 5.14 step 8 is skipped, we only support configuring
@ SDRAM out of RESET


@ ** Dev Manual sect. 5.14 step 9
	lreg r3, MDCNFG		@ sdram config -- sdram enable
	ldr r2, [r3]
	orr r2, r2, #0x00000001	@ enable appropriate banks, value depends on selected banks
	str r2, [r3]			@ write to MDCNFG


@ ** Dev Manual sect. 5.14 step 10
	lreg r3, MDMRS			@ write the MDMRS
	lreg r2, 0x00020032		@ the writable bits will be written as a 0
	str r2, [r3]

@ Return
	mov pc, lr

@********************************************************************************************

